@@ -4,7 +4,7 @@ cms <- function(rwl, po, c.hat.t=FALSE, c.hat.i=FALSE) {
         tt <- theDat[, 1]
         n <- nrow(theDat)
         err4 <- array(0, n)
-        for (i in c(1:n)){
+        for (i in 1:n){
             theDat.2 <- theDat[i, 2]
             err1 <- theDat.2^4
             tt.i <- tt[i]
@@ -18,32 +18,33 @@ cms <- function(rwl, po, c.hat.t=FALSE, c.hat.i=FALSE) {
         list(indices=err6, c.val=med)
     }
 ### main func
-    if(ncol(rwl) != nrow(po))
+    n.col <- ncol(rwl)
+    if(n.col != nrow(po))
         stop("dimension problem: ncol(rw) != nrow(po)")
-    if(!all(po[, 1] %in% colnames(rwl)))
+    col.names <- colnames(rwl)
+    if(!all(po[, 1] %in% col.names))
         stop("Series ids in 'po' and 'rwl' do not match")
     rownames(rwl) <- rownames(rwl) # guard against NULL names funniness
     series.yrs <- apply(rwl, 2, yr.range)
-    rownames(series.yrs) <- c("first", "last")
 
     rwl.ord <- apply(rwl, 2, sortByIndex)
-    rwca <- data.frame(matrix(NA, ncol=ncol(rwl.ord),
+    rwca <- data.frame(matrix(NA, ncol=n.col,
                               nrow=sum(nrow(rwl.ord) + max(po[, 2]))))
-    colnames(rwca) <- colnames(rwl)
-    for (i in 1:ncol(rwl.ord)){
-        series <- colnames(rwl.ord)[i]
-        yrs2pith <- po[po[, 1] %in% series, 2]
-        rwca[(yrs2pith):(yrs2pith + nrow(rwl.ord)-1), i] <- rwl.ord[, i]
+    colnames(rwca) <- col.names
+    nrow.m1 <- nrow(rwl.ord) - 1
+    for (i in 1:n.col){
+        yrs2pith <- po[po[, 1] %in% col.names[i], 2]
+        rwca[yrs2pith:(yrs2pith + nrow.m1), i] <- rwl.ord[, i]
     }
 
     ## divide each series by c curve and restore to cal years
     rwi <- rwl
-    c.vec <- rep(NA, ncol(rwi))
-    names(c.vec) <- colnames(rwca)
+    c.vec <- rep(NA, n.col)
+    names(c.vec) <- col.names
     c.curve.df <- rwca
-    c.curve.df[, 1:ncol(c.curve.df)] <- NA
+    c.curve.df[, ] <- NA
     yrs <- as.numeric(rownames(rwi))
-    for(i in 1:ncol(rwca)){
+    for(i in 1:n.col){
         no.na <- which(!is.na(rwca[, i]))
         index <- cbind(no.na, rwca[no.na, i])
         tmp <- biologicalTrend(index)
@@ -51,10 +52,9 @@ cms <- function(rwl, po, c.hat.t=FALSE, c.hat.i=FALSE) {
         c.curve <- tmp[[1]]
         c.curve.df[1:(po[i, 2]+length(c.curve)), i] <-
             c(rep(NA, po[i, 2]), c.curve)
-        y <- rwca[no.na, i] / c.curve
         first <- series.yrs[1, i]
         last <- series.yrs[2, i]
-        rwi[yrs %in% first:last, i] <- y
+        rwi[yrs %in% first:last, i] <- rwca[no.na, i] / c.curve
     }
     ## export options
     if(c.hat.t) {
